from pytwss.mbean.mbean_utils import MBeanProxy, Scope, MAX_RETRIES, getAdminClient
from pytwss.mbean.discovery_mbean import DiscoveryAdministration
from pytwss.utils import _, log, logScreen
import jarray, javax
from time import sleep
from java.lang import Boolean, System
from java.util.concurrent.locks import ReentrantLock
from java.util.concurrent import TimeUnit

class DiscoveryListener(javax.management.NotificationListener):
    def handleNotification(self, notification, handback):
       notification_type = notification.getType()
       log("Recieved notification %s." % notification_type)

       if notification_type == 'websphere.discovery.process.found':
          # Obtain the lock.
          node_lock.lock()
          try:
             # Signal that the node has been restarted.
             node_condition.signalAll()
          finally:
             # Release the lock.
             node_lock.unlock()


# The node_lock and node_condition wait for a node agent to be restarted.
node_lock = ReentrantLock()
node_condition = node_lock.newCondition()
dl = DiscoveryListener()

#
# The MBeanProxy to the NodeAgent mbean.
#
class NodeAgentAdministration(MBeanProxy):
    # Constructs a NodeAgentAdministration object.
    # param: nodeName the name of the node of the node agent.
    # param: max_retries max retry count to connect to mbean
    def __init__(self, node_name, max_retries=MAX_RETRIES):
       scope = Scope(type='NodeAgent', node=node_name)
       MBeanProxy.__init__(self, scope, max_retries=max_retries)

#
# Restarts a node agent using the NodeAgent mbean.
# 
# param: nodeName the names of the node to restart
# param: maxWait max time to wait for restart, default is 300 secs (5 minutes)
# return: the NodeAgentInfo
#
def restartNodeAgent(nodeName, maxWait=300):

    ac = getAdminClient()
    discovery = DiscoveryAdministration()
    nmgr = NodeAgentAdministration(nodeName)
    
    # Add a listener for the node agent.
    log("Adding listener for NodeAgent Mbean %s." % nodeName)
    ac.addNotificationListener(discovery.ref, dl, None, None)

    # Obtain the lock.
    node_lock.lock()
    
    try:
    
       # Get the deadline for waiting.
       deadline = maxWait * 1000
       
       #Restart the node agent.
       log("Restarting node agent: %s." % nodeName)
       logScreen(_("SOAC7534I") %{ '1' : nodeName })
       nmgr.restart(Boolean(1), Boolean(0))
        
       # Wait for the node agent to restart.
       is_restarted = node_condition.await(deadline, TimeUnit.MILLISECONDS)
       # Check if the node agent restarted.
       if is_restarted:
          log("Node agent (%s) is started." % nodeName)
          logScreen(_("SOAC7536I") %{ '1' : nodeName })
       else:
          log("The Node Agent (%s) failed to restart before the deadline." % nodeName)
     
    finally:
       # Release the lock.
       node_lock.unlock()
   
       #Remove the listener
       log("Removing listener for NodeAgent Mbean %s." % nodeName)
       ac.removeNotificationListener(discovery.ref, dl)


#
# Gets the node agent info from the NodeAgent mbean.
# Query the NodeAgentInfo to determine if all the node agents are running
# for every node defined in the cluster.
#
# param: nodeNames the names of all of the nodes in the cluster
# return: the NodeAgentInfo
#
def getNodeAgentInfo(nodeNames):
   isRunning = "true"
   runningNodeAgents = []
   stoppedNodeAgents = []
   for node in nodeNames:
      nmgr = NodeAgentAdministration(node)
      if nmgr.ref:
         runningNodeAgents.append(node)
      else:
         isRunning = "false"
         stoppedNodeAgents.append(node)
   return __NodeAgentInfo(isRunning, runningNodeAgents, stoppedNodeAgents)

#
# __NodeAgentInfo can be queried to retrieve the status of the node agents,
# including the names of the running and stopped node agents.
#
class __NodeAgentInfo:
	#
	# Constructs a __NodeAgentInfo object.
	# param: isRunning 1 if all the node agents are running
	# param: runningNodeAgents list of running node agent names
	# param: stoppedNodeAgents list of stopped node agent names
	#
	def __init__(self, isRunning, runningNodeAgents, stoppedNodeAgents):
   		self.__isRunning = isRunning
		self.__runningNodeAgents = runningNodeAgents
		self.__stoppedNodeAgents = stoppedNodeAgents

	#
	# Gets the status of the entire set of node agents.
	# return: 1 if all the node agents are running.
	#
	def isRunning(self):
		return self.__isRunning

	#
	# Gets a list of names of the running node agents.
	# return: the list of names of the running node agents.
	#
	def getRunningNodeAgents(self):
		return self.__runningNodeAgents
	
	#
	# Gets a list of names of the stopped node agents.
	# return: the list of names of the stopped node agents.
	#
	def getStoppedNodeAgents(self):
		return self.__stoppedNodeAgents

	def toString(self):
		nodeagent_info_str = []
		nodeagent_info_str.append("isRunning: %s" % self.__isRunning)
		nodeagent_info_str.append("Running Agents: %s" % self.__runningNodeAgents)
		nodeagent_info_str.append("Stopped Agents: %s" % self.__stoppedNodeAgents)
		return ", ".join(nodeagent_info_str)
		

   
